from django.contrib import admin
from webleads.models import LeadInfo, ActionType, LeadAction
from django.utils.translation import ugettext_lazy as _


class LeadInfoAdmin(admin.ModelAdmin):
    search_fields = ('full_name', 'email', )
    list_display = ('session_key', 'full_name', 'email', 'score', )


class LeadActionAdmin(admin.ModelAdmin):
    search_fields = ('lead_info__full_name', 'lead_info__email', )
    list_display = ('date', 'lead_info', 'info_user_email', 'comment',
                    'action_type', 'product_key', 'answered', )
    list_filter = ('product_key', 'action_type', 'date', 'answered',)

    def info_user_email(self, obj):
        return obj.lead_info.email
    info_user_email.short_description = _('email')


class ActionTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'score', )


admin.site.register(LeadInfo, LeadInfoAdmin)
admin.site.register(ActionType, ActionTypeAdmin)
admin.site.register(LeadAction, LeadActionAdmin)



from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _


class LeadInfo(models.Model):
    session_key = models.CharField(_('session key'), max_length=40,
                                   primary_key=True)
    full_name = models.CharField(_('full name'), max_length=40, blank=False,
                                null=False)
    email = models.EmailField(_('e-mail address'), blank=False, null=False)
    score = models.PositiveIntegerField(_('score'), default=0, editable=False)

    def __unicode__(self):
        return self.full_name


class ActionType(models.Model):
    slug = models.SlugField(_('action slug'))
    name = models.CharField(_('full name'), max_length=100, blank=False, null=False)
    score = models.PositiveIntegerField(_('score'))

    def __unicode__(self):
        return self.name


class LeadAction(models.Model):
    lead_info = models.ForeignKey(LeadInfo)
    product_key = models.SlugField(_('product key'))
    comment = models.TextField(_('Comment'), null=True, blank=True)
    action_type = models.ForeignKey(ActionType)
    date = models.DateTimeField(auto_now_add=True)
    phone = models.CharField(_('phone'), max_length=20, blank=True,
                            null=True)
    answered = models.BooleanField(_('awnswer sent'))

    class Meta:
        ordering = ('-date', )

    def __unicode__(self):
        return u'%s (%s)' % (self.action_type, self.comment)


def handle_saved_action(sender, **kwargs):
    lead_action, created = kwargs['instance'], kwargs['created']
    if created:
        # add the score to de weblead, depending on the action type
        lead_action.lead_info.score += lead_action.action_type.score
        lead_action.lead_info.save()


signals.post_save.connect(handle_saved_action, sender=LeadAction)



from functools import wraps


def lead_action(action_type_slug=None, product_key=None):
    """
    Decorator that executes an action for the lead, which will increase the lead
    scoring.

    The action executed will be a new action of the passed ``action_type_slug``,
    and will be recorded for ``product_key`` product.

    Usage::

        from django.http import HttpResponse

        from webleads.decorators import lead_action

        @lead_action('demo-request', 'uniquid')
        def do_demo_request(request):
            return HttpResponse('demo submitted')
    """
    def _lead_action(viewfunc):
        def _lead_action_inner(request, *args, **kw):
            response = viewfunc(request, *args, **kw)
            from webleads.models import ActionType, LeadAction
            action_type = ActionType.objects.get(slug=action_type_slug)
            LeadAction.objects.create(
                action_type=action_type,
                comment='',
                lead_info=request.lead_info,
                product_key=product_key,
            )
            return response
        return wraps(viewfunc)(_lead_action_inner)
    return _lead_action



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-webleads.
#
# django-webleads is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-webleads is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-webleads.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


def parse_requirements(file_name):
    requirements = []
    for line in open(file_name, 'r').read().split('\n'):
        if re.match(r'(\s*#)|(\s*$)', line):
            continue
        if re.match(r'\s*-e\s+', line):
            requirements.append(re.sub(r'\s*-e\s+.*#egg=(.*)$', r'\1', line))
        elif re.match(r'\s*-f\s+', line):
            pass
        else:
            requirements.append(line)

    return requirements


def parse_dependency_links(file_name):
    dependency_links = []
    for line in open(file_name, 'r').read().split('\n'):
        if re.match(r'\s*-[ef]\s+', line):
            dependency_links.append(re.sub(r'\s*-[ef]\s+', '', line))

    return dependency_links


data_files = []
for dirpath, dirnames, filenames in os.walk('.'):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        continue
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

setup(
    name="django-webleads",
    version="0.2b",
    install_requires=parse_requirements('requirements.txt'),
    dependency_links=parse_dependency_links('requirements.txt'),
    author="Manuel Saelices",
    author_email="msaelices@yaco.es",
    description="Django application to manage incoming leads to your website",
    long_description=(read('README') + '\n\n' + read('CHANGES')),
    url="",
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
        'Programming Language :: Python',
    ],
    packages=find_packages(),
    data_files=data_files,
)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'LeadAction.phone'
        db.add_column('webleads_leadaction', 'phone',
                      self.gf('django.db.models.fields.CharField')(max_length=20, null=True, blank=True),
                      keep_default=False)

        # Adding field 'LeadAction.answered'
        db.add_column('webleads_leadaction', 'answered',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'LeadAction.phone'
        db.delete_column('webleads_leadaction', 'phone')

        # Deleting field 'LeadAction.answered'
        db.delete_column('webleads_leadaction', 'answered')

    models = {
        'webleads.actiontype': {
            'Meta': {'object_name': 'ActionType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'score': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'})
        },
        'webleads.leadaction': {
            'Meta': {'ordering': "('-date',)", 'object_name': 'LeadAction'},
            'action_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['webleads.ActionType']"}),
            'answered': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lead_info': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['webleads.LeadInfo']"}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'product_key': ('django.db.models.fields.SlugField', [], {'max_length': '50'})
        },
        'webleads.leadinfo': {
            'Meta': {'object_name': 'LeadInfo'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'score': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True'})
        }
    }

    complete_apps = ['webleads']



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.core.mail import send_mail
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site

from webleads.models import LeadAction, ActionType


class LeadActionForm(forms.Form):
    full_name = forms.CharField(max_length=100, label=_('Full Name'))
    email = forms.EmailField()
    phone = forms.CharField(label=_('phone'), required=False, max_length=20)

    def __init__(self, request, product_key, data=None, *args, **kwargs):
        super(LeadActionForm, self).__init__(data, *args, **kwargs)
        self.request = request
        self.lead_info = request.lead_info
        self.product_key = product_key
        if data is None:
            # obtaining data from weblead if have been saved previously
            lead_full_name = self.lead_info.full_name
            lead_email = self.lead_info.email
            if lead_full_name:
                self.fields['full_name'].initial = lead_full_name
            if lead_email:
                self.fields['email'].initial = lead_email

    def base_save_action(self, action_slug, product=None, comment=None):
        action_type = ActionType.objects.get(slug=action_slug)
        LeadAction.objects.create(
            action_type=action_type,
            comment=comment,
            lead_info=self.lead_info,
            product_key=self.product_key,
        )

    def save_action(self):
        raise NotImplementedError()

    def save(self):
        self.lead_info.full_name = self.cleaned_data['full_name']
        self.lead_info.email = self.cleaned_data['email']
        self.lead_info.save()
        self.save_action()

    def _send_mail(self, comment, trans_msg):
        send_mail(
            '%s (%s)' % (comment, self.product_key),
            trans_msg % {
                'lead': self.lead_info.full_name,
                'url': 'http://%s/admin/webleads/leadaction/' % Site.objects.get_current().domain,
            },
            settings.DEFAULT_FROM_EMAIL,
            [settings.MARKETING_EMAIL],
            fail_silently=False,
        )

    def _customer_mail(self, trans_msg):
        send_mail(
            ugettext('This is the product that you request more info'),
            trans_msg % {
                'product': self.product_key,
            },
            settings.DEFAULT_FROM_EMAIL,
            [self.lead_info.email],
            fail_silently=False,
        )


class DemoRequestForm(LeadActionForm):

    def save_action(self):
        comment = ugettext('Demo request sent from web')
        self.base_save_action(
            action_slug='demo-request',
            comment=comment,
        )
        self._send_mail(comment, ugettext('There is a new demo request sent by %(lead)s. More details in %(url)s'))


class ContactRequestForm(LeadActionForm):

    def save_action(self):
        comment = ugettext('Contact request sent from web')
        self.base_save_action(
            action_slug='contact-request',
            comment=comment,
        )
        self._send_mail(comment, ugettext('There is a new contact request sent by %(lead)s. More details in %(url)s'))


class BrochureRequestForm(LeadActionForm):

    def save_action(self):
        comment = ugettext('Brochure download request sent from web')
        self.base_save_action(
            action_slug='catalog-download',
            comment=comment,
        )
        self._send_mail(comment, ugettext('There is a new brochure download request sent by %(lead)s. More details in %(url)s'))
        self._customer_mail(ugettext('You request about %(product)s, we will contact you back as soon as posible. http://www.yaco.es/media/products/%(product)s.pdf'))



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'LeadInfo'
        db.create_table('webleads_leadinfo', (
            ('session_key', self.gf('django.db.models.fields.CharField')(max_length=40, primary_key=True)),
            ('full_name', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('score', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
        ))
        db.send_create_signal('webleads', ['LeadInfo'])

        # Adding model 'ActionType'
        db.create_table('webleads_actiontype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('score', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('webleads', ['ActionType'])

        # Adding model 'LeadAction'
        db.create_table('webleads_leadaction', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('lead_info', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['webleads.LeadInfo'])),
            ('product_key', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('comment', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('action_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['webleads.ActionType'])),
            ('date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('webleads', ['LeadAction'])

    def backwards(self, orm):
        # Deleting model 'LeadInfo'
        db.delete_table('webleads_leadinfo')

        # Deleting model 'ActionType'
        db.delete_table('webleads_actiontype')

        # Deleting model 'LeadAction'
        db.delete_table('webleads_leadaction')

    models = {
        'webleads.actiontype': {
            'Meta': {'object_name': 'ActionType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'score': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'})
        },
        'webleads.leadaction': {
            'Meta': {'ordering': "('-date',)", 'object_name': 'LeadAction'},
            'action_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['webleads.ActionType']"}),
            'comment': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lead_info': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['webleads.LeadInfo']"}),
            'product_key': ('django.db.models.fields.SlugField', [], {'max_length': '50'})
        },
        'webleads.leadinfo': {
            'Meta': {'object_name': 'LeadInfo'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'score': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True'})
        }
    }

    complete_apps = ['webleads']
